@@ -1,9 +1,11 @@
 import numpy as np
+from warnings import warn
+import logging
 from scipy.integrate._ivp.rk import norm, SAFETY, MAX_FACTOR, MIN_FACTOR
 from extensisq.common import RungeKutta, HornerDenseOutput, NFS
 
 
-class BS45(RungeKutta):
+class BS5(RungeKutta):
     """Explicit Runge-Kutta method of order 5(4).
 
     This uses the Bogacki-Shampine pair of formulas [1]_. It is designed
@@ -55,6 +57,27 @@ class BS45(RungeKutta):
     vectorized : bool, optional
         Whether `fun` is implemented in a vectorized fashion. A vectorized
         implementation offers no advantages for this solver. Default is False.
+    nfev_stiff_detect : int, optional
+        Number of function evaluations for stiffness detection. This number has
+        multiple purposes. If it is set to 0, then stiffness detection is
+        disabled. For other (positive) values it is used to represent a
+        'considerable' number of function evaluations (nfev). A stiffness test
+        is done if many steps fail and each time nfev exceeds integer multiples
+        of `nfev_stiff_detect`. For the assessment itself, the problem is
+        assessed as non-stiff if the predicted nfev to complete the integration
+        is lower than `nfev_stiff_detect`. The default value is 5000.
+    interpolant : 'best', 'low' or 'free', optional
+        Select the interpolant for dense output. The option 'best' is for the
+        accurate fifth order interpolant described in [1], which needs 3 extra
+        function evaluations per step. The option 'low' is for a less accurate
+        fifth order interpolant which needs 'only' one extra function
+        evaluation per step. 'free' is for a free fourth order interpolant.
+        Recommendations: 'best' for events, 'free' for simple plotting or for
+        long integrations for which the gobal error dominates. The accuracy of
+        the method itself does not change and the extra function evaluations
+        are only done at steps for which dense output is requested. The default
+        is 'low': a safe option for which the method does not loose much of its
+        performance when it is used with dense output.
 
     Attributes
     ----------
@@ -93,8 +116,10 @@ class BS45(RungeKutta):
 
     order = 5
     error_estimator_order = 4
-    n_stages = 7        # the effective nr (total nr of stages is 8)
-    n_extra_stages = 3  # for dense output
+    n_stages = 7            # the effective nr (total nr of stages is 8)
+    n_extra_stages = 3      # for dense output
+    tanang = 5.2
+    stbrad = 3.9
 
     # time step fractions
     C = np.array([0, 1/6, 2/9, 3/7, 2/3, 3/4, 1])
@@ -121,15 +146,14 @@ class BS45(RungeKutta):
     # coefficients for main error estimation method
     E = np.array([2479/34992, 0, 123/416, 612941/3411720, 43/1440,
                   2272/6561, 79937/1113912, 3293/556956])
-    E[:-1] -= B   # convert to error coefficients
+    E[:-1] -= B     # convert to error coefficients
 
     # extra time step fractions for dense output
     C_extra = np.array([1/2, 5/6, 1/9])
 
     # coefficient matrix for dense output
     A_extra = np.array([
-        [455/6144, -837888343715/13176988637184,
-            98719073263/1551965184000],
+        [455/6144, -837888343715/13176988637184, 98719073263/1551965184000],
         [0, 30409415/52955362, 1307/123552],
         [10256301/35409920, -48321525963/759168069632,
             4632066559387/70181753241600],
@@ -142,8 +166,9 @@ class BS45(RungeKutta):
         [0, -5183640/26477681, -1/10],
         [0, 0, -1403317093/11371610250]]).T
 
-    # coefficients for interpolation (high order, default)
-    P = np.array([
+    # This is the accurate, 5th order interpolant described of [1, 2].
+    # It needs 3 extra function evaluations per step.
+    Pbest = np.array([
         [0, -11513270273/3502699200, -87098480009/5254048800,
             -2048058893/59875200, -1620741229/50038560,
             -12134338393/1050809760],
@@ -166,13 +191,58 @@ class BS45(RungeKutta):
             -1774004627/25270245, -1774004627/75810735],
         [0, 12, 59, 117, 105, 35]])
 
-    def __init__(self, fun, t0, y0, t_bound, **extraneous):
-        super(BS45, self).__init__(fun, t0, y0, t_bound, **extraneous)
+    # This is an up to 12x less accurate, 5th order interpolant that needs one
+    # extra function evaluation per step, instead of three.
+    Plow = np.array([
+        [1, -155/36, 16441/2016, -56689/8064, 757/336],
+        [0, 0, 0, 0, 0],
+        [0, 6561/988, -14727987/774592, 60538347/3098368, -13321017/1936480],
+        [0, 2401/702, -603337/56160, 2740913/224640, -24353/5200],
+        [0, 0, -387/2240, 3483/8960, -1161/5600],
+        [0, 1408/513, -45536/3591, 67960/3591, -17216/1995],
+        [0, 0, -7267/4704, 21801/6272, -7267/3920],
+        [0, -1/2, 4, -15/2, 4],
+        [0, -8, 32, -40, 16]])
+
+    # This is a low accuracy, 4th order interpolant. It is free: it does not
+    # need extra function evaluations.
+    P = np.array([
+        [1, -2773674729811/735370896960, 316222661411/52526492640,
+            -1282818361681/294148358784, 6918746667/5836276960],
+        [0, 0, 0, 0, 0],
+        [0, 1594012432639617/282545840187520, -303081611134977/20181845727680,
+            1643668176796011/113018336075008, -14071997888919/2883120818240],
+        [0, -47637453654133/20485332129600, 125365109861131/10242666064800,
+            -135424370922463/8194132851840, 2582696138393/379358002400],
+        [0, 1915795112337/817078774400, -557453242737/58362769600,
+            3958638678747/326831509760, -285784868817/58362769600],
+        [0, -1490252641456/654939705105, 692325952352/93562815015,
+            -808867306376/130987941021, 4887837472/3465289445],
+        [0, 824349534931/571955142080, -895925604353/122561816160,
+            2443928282393/228782056832, -5528580993/1167255392],
+        [0, -38480331/36476731, 226874786/36476731,
+            -374785310/36476731, 186390855/36476731]])
+
+    def __init__(self, fun, t0, y0, t_bound, nfev_stiff_detect=5000,
+                 interpolant='low', **extraneous):
+        super(BS5, self).__init__(
+            fun, t0, y0, t_bound, nfev_stiff_detect=nfev_stiff_detect,
+            **extraneous)
         # custom initialization to create extended storage for dense output
-        # and to make the interpolator selectable
-        self.K_extended = np.zeros((self.n_stages+self.n_extra_stages+1,
-                                    self.n), dtype=self.y.dtype)
-        self.K = self.K_extended[:self.n_stages+1]
+        if interpolant not in ('best', 'low', 'free'):
+            raise ValueError(
+                "interpolant should be one of: 'best', 'low', 'free'")
+        self.interpolant = interpolant
+        if self.interpolant == 'best':
+            self.K_extended = np.zeros((self.n_stages+self.n_extra_stages + 1,
+                                        self.n), dtype=self.y.dtype)
+            self.K = self.K_extended[:self.n_stages+1]
+        elif self.interpolant == 'low':
+            self.K_extended = np.zeros((self.n_stages + 2,
+                                        self.n), dtype=self.y.dtype)
+            self.K = self.K_extended[:self.n_stages+1]
+        else:
+            self.K_extended = self.K
         # y_old is used for first error assessment, it should not be None
         self.y_old = self.y - self.direction * self.h_abs * self.f
 
@@ -183,24 +253,36 @@ class BS45(RungeKutta):
         y = self.y
 
         # limit step size
-        min_step = 10 * np.abs(np.nextafter(t, self.direction * np.inf) - t)
-        h_abs = min(self.max_step, max(min_step, self.h_abs))
+        h_abs = self.h_abs
+        min_step = max(self.h_min_a * (abs(t) + h_abs), self.h_min_b)
+        h_abs = min(self.max_step, max(min_step, h_abs))
 
-        # use extrapolation in the rare cases where the previous step ended
-        # too close to t_bound.
-        d = abs(self.t_bound - t)
-        if d < min_step:
-            h = self.t_bound - t
-            y_new = y + h * self.f
-            self.h_previous = h
-            self.y_old = y
-            self.t = self.t_bound
-            self.y = y_new
-            self.f = None                          # used by _dense_output_impl
-            return True, None
-
-        # don't step over t_bound
-        h_abs = min(h_abs, d)
+        # handle final integration steps
+        d = abs(self.t_bound - t)               # remaining interval
+        if d < 2 * h_abs:
+            if d >= min_step:
+                if h_abs < d:
+                    # h_abs < d < 2 * h_abs:
+                    # split d over last two steps ("look ahead").
+                    # This reduces the chance of a very small last step.
+                    h_abs = max(0.5 * d, min_step)
+                else:
+                    # d <= h_abs:
+                    # don't step over t_bound
+                    h_abs = d
+            else:
+                # d < min_step:
+                # use linear extrapolation in this rare case
+                h = self.t_bound - t
+                y_new = y + h * self.f
+                self.h_previous = h
+                self.y_old = y
+                self.t = self.t_bound
+                self.y = y_new
+                self.f = None                    # to signal _dense_output_impl
+                logging.warning(
+                    'Linear extrapolation was used in the final step.')
+                return True, None
 
         # loop until step accepted
         step_accepted = False
@@ -262,13 +344,22 @@ class BS45(RungeKutta):
         # output
         self.t = t_new
         self.y = y_new
+
+        # stiffness detection
+        if self.nfev_stiff_detect:
+            self.havg = 0.9 * self.havg + 0.1 * h     # exp moving average
+            self._stiff()
+            self.okstp += 1
+            if self.okstp == 20:
+                self.havg = h
+                self.jflstp = 0
+
         return True, None
 
     def _estimate_error_norm_pre(self, y, h):
         # first error estimate
         # y_new is not available yet for scale, so use y_old instead
-        scale = self.atol + self.rtol * np.maximum(
-            np.abs(y), np.abs(self.y_old))
+        scale = self.atol + self.rtol * 0.5*(np.abs(y) + np.abs(self.y_old))
         err = h * (self.K[:6, :].T @ self.E_pre)
         return norm(err / scale)
 
@@ -276,354 +367,62 @@ class BS45(RungeKutta):
         h = self.h_previous
         K = self.K_extended
 
+        if self.interpolant == 'free':
+            Q = K.T @ self.P
+            return HornerDenseOutput(self.t_old, self.t, self.y_old, Q)
+
+        elif self.interpolant == 'low':
+            s = self.n_stages + 1
+            dy = K[:s, :].T @ self.A_extra[0, :s] * h
+            K[s] = self.fun(self.t_old + self.C_extra[0] * h, self.y_old + dy)
+            Q = K.T @ self.Plow
+            return HornerDenseOutput(self.t_old, self.t, self.y_old, Q)
+
+        # else: the accurate interpolant
         # calculate the required extra stages
         for s, (a, c) in enumerate(zip(self.A_extra, self.C_extra),
                                    start=self.n_stages+1):
             dy = K[:s, :].T @ a[:s] * h
             K[s] = self.fun(self.t_old + c * h, self.y_old + dy)
 
-        # form Q. Usually: Q = K.T @ self.P
-        # but rksuite recommends to group summations to mitigate round-off:
-        Q = np.empty((K.shape[1], self.P.shape[1]), dtype=K.dtype)
+        # form Q. Usually: Q = K.T @ self.P as for the other interpolants,
+        # but RKSuite groups summations to mitigate round-off:
+        Q = np.empty((K.shape[1], self.Pbest.shape[1]), dtype=K.dtype)
         Q[:, 0] = self.K[7]
-        KP = K * self.P[:, 1, np.newaxis]                       # term for t**2
+        KP = K * self.Pbest[:, 1, np.newaxis]                   # term for t**2
         Q[:, 1] = (KP[4] + ((KP[5] + KP[7]) + KP[0]) + ((KP[2] + KP[8]) +
                    KP[9]) + ((KP[3] + KP[10]) + KP[6]))
-        KP = K * self.P[:, 2, np.newaxis]                       # term for t**3
+        KP = K * self.Pbest[:, 2, np.newaxis]                   # term for t**3
         Q[:, 2] = (KP[4] + KP[5] + ((KP[2] + KP[8]) + (KP[9] + KP[7]) +
                    KP[0]) + ((KP[3] + KP[10]) + KP[6]))
-        KP = K * self.P[:, 3, np.newaxis]                       # term for t**4
+        KP = K * self.Pbest[:, 3, np.newaxis]                   # term for t**4
         Q[:, 3] = (((KP[3] + KP[7]) + (KP[6] + KP[5]) + KP[4]) + ((KP[9] +
                    KP[8]) + (KP[2]+KP[10]) + KP[0]))
-        KP = K * self.P[:, 4, np.newaxis]                       # term for t**5
+        KP = K * self.Pbest[:, 4, np.newaxis]                   # term for t**5
         Q[:, 4] = ((KP[9] + KP[8]) + ((KP[6] + KP[5]) + KP[4]) + ((KP[3] +
                    KP[7]) + (KP[2] + KP[10]) + KP[0]))
-        KP = K * self.P[:, 5, np.newaxis]                       # term for t**6
+        KP = K * self.Pbest[:, 5, np.newaxis]                   # term for t**6
         Q[:, 5] = (KP[4] + ((KP[9] + KP[7]) + (KP[6] + KP[5])) + ((KP[3] +
                    KP[8]) + (KP[2] + KP[10]) + KP[0]))
 
-        # Rksuite uses Horner's rule to evaluate the polynomial. Moreover,
-        # the polynomial definition is different: looking back from the end
-        # of the step instead of forward from the start.
+        # RKSuite's, polynomial definition is different from scipy's: looking
+        # back from the end of the step instead of forward from the start.
         # The call is modified to accomodate:
         return HornerDenseOutput(self.t, self.t+h, self.y, Q)
 
 
-class BS45_i(BS45):
-    """As BS45, but with free 4th order interpolant for dense output. Suffix _i
-    for interpolant.
-
-    The source [1]_ refers to the thesis of Bogacki for a free interpolant, but
-    this could not be found. Instead, the interpolant is constructed following
-    the steps in [3]_.
-
-    For the best accuracy use BS45 instead. This BS45_i method can be more
-    efficient for several problems if dense_output is used.
-
-    Parameters
-    ----------
-    fun : callable
-        Right-hand side of the system. The calling signature is ``fun(t, y)``.
-        Here ``t`` is a scalar, and there are two options for the ndarray
-        ``y``: It can either have shape (n,); then ``fun`` must return
-        array_like with shape (n,). Alternatively it can have shape (n, k);
-        then ``fun`` must return an array_like with shape (n, k), i.e., each
-        column corresponds to a single column in ``y``. The choice between the
-        two options is determined by `vectorized` argument (see below).
-    t0 : float
-        Initial time.
-    y0 : array_like, shape (n,)
-        Initial state.
-    t_bound : float
-        Boundary time - the integration won't continue beyond it. It also
-        determines the direction of the integration.
-    first_step : float or None, optional
-        Initial step size. Default is ``None`` which means that the algorithm
-        should choose.
-    max_step : float, optional
-        Maximum allowed step size. Default is np.inf, i.e., the step size is
-        not bounded and determined solely by the solver.
-    rtol, atol : float and array_like, optional
-        Relative and absolute tolerances. The solver keeps the local error
-        estimates less than ``atol + rtol * abs(y)``. Here `rtol` controls a
-        relative accuracy (number of correct digits). But if a component of `y`
-        is approximately below `atol`, the error only needs to fall within
-        the same `atol` threshold, and the number of correct digits is not
-        guaranteed. If components of y have different scales, it might be
-        beneficial to set different `atol` values for different components by
-        passing array_like with shape (n,) for `atol`. Default values are
-        1e-3 for `rtol` and 1e-6 for `atol`.
-    vectorized : bool, optional
-        Whether `fun` is implemented in a vectorized fashion. A vectorized
-        implementation offers no advantages for this solver. Default is False.
-
-    Attributes
-    ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
-    t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
-
-    References
-    ----------
-    .. [1] P. Bogacki, L.F. Shampine, "An efficient Runge-Kutta (4,5) pair",
-           Computers & Mathematics with Applications, Vol. 32, No. 6, 1996,
-           pp. 15-28, ISSN 0898-1221.
-           https://doi.org/10.1016/0898-1221(96)00141-1
-    .. [2] RKSUITE: https://www.netlib.org/ode/rksuite/
-    .. [3] Ch. Tsitouras, "Runge-Kutta pairs of order 5(4) satisfying only the
-           first column simplifying assumption", Computers & Mathematics with
-           Applications, Vol. 62, No. 2, pp. 770 - 775, 2011.
-           https://doi.org/10.1016/j.camwa.2011.06.002
-    """
-
-    # Bogacki published a free interpolant in his thesis, but I was not able to
-    # find a copy of it. Instead, I constructed an interpolant using sympy and
-    # the approach in [3]_.
-    # This free 4th order interpolant has a leading error term ||T5|| that has
-    # maximum in [0,1] of 5.47 e-4. This is higher than the corresponding term
-    # of the embedded fourth order method: 1.06e-4.
-
-    # overwrite P
-    P = np.array([
-        [1, -2773674729811/735370896960,
-            316222661411/52526492640,
-            -1282818361681/294148358784,
-            6918746667/5836276960],
-        [0, 0, 0, 0, 0],
-        [0, 1594012432639617/282545840187520,
-            -303081611134977/20181845727680,
-            1643668176796011/113018336075008,
-            -14071997888919/2883120818240],
-        [0, -47637453654133/20485332129600,
-            125365109861131/10242666064800,
-            -135424370922463/8194132851840,
-            2582696138393/379358002400],
-        [0, 1915795112337/817078774400,
-            -557453242737/58362769600,
-            3958638678747/326831509760,
-            -285784868817/58362769600],
-        [0, -1490252641456/654939705105,
-            692325952352/93562815015,
-            -808867306376/130987941021,
-            4887837472/3465289445],
-        [0, 824349534931/571955142080,
-            -895925604353/122561816160,
-            2443928282393/228782056832,
-            -5528580993/1167255392],
-        [0, -38480331/36476731,
-            226874786/36476731,
-            -374785310/36476731,
-            186390855/36476731]])
-
-    def __init__(self, fun, t0, y0, t_bound, **extraneous):
-        super(BS45, self).__init__(fun, t0, y0, t_bound, **extraneous)
-        # y_old is used for first error assessment, it should not be None
-        self.y_old = self.y - self.direction * self.h_abs * self.f
-
-    def _dense_output_impl(self):
-        return super(BS45, self)._dense_output_impl()
+# old class names
+class BS45_i(BS5):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'BS5'. "
+             "Its option 'interpolant' selects the interpolant",
+             FutureWarning)
+        super(BS45_i, self).__init__(*args, **kwargs, interpolant="free")
 
 
-if __name__ == '__main__':
-    """Construction of a free interpolant of the BS45 pair. The approach from
-    "Runge-Kutta pairs of order 5(4) satisfying only the first column
-    simplifying assumption" by Ch Tsitouras is followed.
-    Bogacki has derived an interpolant for this method as well, but I was not
-    able to find a copy of his thesis that contains this interpolant.
-    """
-    # import numpy as np
-    import matplotlib.pyplot as plt
-    import sympy
-    from sympy.solvers.solveset import linsolve
-    from sympy import Rational as R
-    from pprint import pprint
-
-    n_stages = 8            # including derivative evaluation at end of step
-    order = 5               # of interpolation in t (not h)
-    T5_method4 = 1.06e-4    # error of embedded fourth order method
-
-    t = sympy.symbols('t', real=True)
-    bi = sympy.symbols(f'bi0:{n_stages}', real=True)
-    bi_vec = sympy.Matrix(bi)
-
-    # Method
-    A = sympy.Matrix([      # full A matrix, including last line
-        [0, 0, 0, 0, 0, 0, 0, 0],
-        [R(1, 6), 0, 0, 0, 0, 0, 0, 0],
-        [R(2, 27), R(4, 27), 0, 0, 0, 0, 0, 0],
-        [R(183, 1372), R(-162, 343), R(1053, 1372), 0, 0, 0, 0, 0],
-        [R(68, 297), R(-4, 11), R(42, 143), R(1960, 3861), 0, 0, 0, 0],
-        [R(597, 22528), R(81, 352), R(63099, 585728), R(58653, 366080),
-         R(4617, 20480), 0, 0, 0],
-        [R(174197, 959244), R(-30942, 79937), R(8152137, 19744439),
-         R(666106, 1039181), R(-29421, 29068), R(482048, 414219), 0, 0],
-        [R(587, 8064), 0, R(4440339, 15491840), R(24353, 124800),
-         R(387, 44800), R(2152, 5985), R(7267, 94080), 0]])
-
-    c = sympy.Matrix([0, R(1, 6), R(2, 9), R(3, 7), R(2, 3), R(3, 4), 1, 1])
-    e = sympy.Matrix([1, 1, 1, 1, 1, 1, 1, 1])
-
-    # error terms up to order 4
-    c2 = c.multiply_elementwise(c)
-    Ac = A*c
-    c3 = c2.multiply_elementwise(c)
-    cAc = c.multiply_elementwise(Ac)
-    Ac2 = A*c2
-    A2c = A*Ac
-
-    T11 = bi_vec.dot(e) - t
-    T21 = bi_vec.dot(c) - t**2/2
-
-    T31 = bi_vec.dot(c2)/2 - t**3/6
-    T32 = bi_vec.dot(Ac) - t**3/6
-
-    T41 = bi_vec.dot(c3)/6 - t**4/24
-    T42 = bi_vec.dot(cAc) - t**4/8
-    T43 = bi_vec.dot(Ac2)/2 - t**4/24
-    T44 = bi_vec.dot(A2c) - t**4/24
-
-    # solve polynomials to let all terms up to order 4 vanish
-    bi_vec_t = sympy.Matrix(linsolve([
-        T11, T21, T31, T32, T41, T42, T43, T44], bi).args[0])
-    i_free_poly = [i for i, (term, poly) in enumerate(zip(bi_vec_t, bi))
-                   if poly == term]
-    free_polys = [bi[i] for i in i_free_poly]
-    print('free polynomials:', free_polys)      # polys bi5 and bi_7 are free
-
-    # Make these free polynommials explicit in t
-    parameters = sympy.symbols([f'bi{i}_0:{order + 1}' for i in i_free_poly])
-    polys = []
-    for coefs in parameters:
-        p = 0
-        for i, coef in enumerate(coefs):
-            p = p + coef * t**i
-        polys.append(p)
-
-    # substitute in bi_vec_t
-    subs_dict = dict(zip(free_polys, polys))
-    bi_vec_t = bi_vec_t.subs(subs_dict)
-
-    # demand continuity at start and end of step
-    d_bi_vec_t = sympy.diff(bi_vec_t, t)        # derivative
-    # C0 at t=0
-    C0_0 = [eq for eq in bi_vec_t.subs(t, 0)]
-    # C0 at t=1
-    C0_1 = [eq for eq in (bi_vec_t.subs(t, 1) - A[-1, :].T)]
-    # C1 at t=0
-    C1_0 = d_bi_vec_t.subs(t, 0)
-    C1_0[0] = C1_0[0] - 1
-    C1_0 = [eq for eq in C1_0]
-    # C1 at t=1
-    C1_1 = d_bi_vec_t.subs(t, 1)
-    C1_1[-1] = C1_1[-1] - 1
-    C1_1 = [eq for eq in C1_1]
-
-    # combine equations in list
-    eqns = C0_0
-    eqns.extend(C0_1)
-    eqns.extend(C1_0)
-    eqns.extend(C1_1)
-
-    # combine parameters in list
-    params = []
-    for p in parameters:
-        params.extend(p)
-
-    # solve continuity constraints
-    sol1 = linsolve(eqns, params).args[0]
-
-    # whych params are still free?
-    free_params = [p for s, p in zip(sol1, params) if s == p]
-    print('free parameters:', free_params)
-    # remaining free parameters: bi5_4, bi5_5, bi7_4, bi7_5
-
-    # update bi_vec_t
-    subs_dict = dict(zip(params, sol1))
-    bi_vec_t = bi_vec_t.subs(subs_dict)
-
-    # find value for free parameter that minimizes the 5th order error terms
-
-    # error terms of order 5
-    c4 = c3.multiply_elementwise(c)
-    c2Ac = c2.multiply_elementwise(Ac)
-    Ac_Ac = Ac.multiply_elementwise(Ac)
-    cAc2 = c.multiply_elementwise(Ac2)
-    Ac3 = A*c3
-    cA2c = c.multiply_elementwise(A2c)
-    A_cAc = A*cAc
-    A2c2 = A*Ac2
-    A3c = A*A2c
-
-    T51 = bi_vec_t.dot(c4)/24 - t**5/120
-    T52 = bi_vec_t.dot(c2Ac)/2 - t**5/20
-    T53 = bi_vec_t.dot(Ac_Ac)/2 - t**5/40
-    T54 = bi_vec_t.dot(cAc2)/2 - t**5/30
-    T55 = bi_vec_t.dot(Ac3)/6 - t**5/120
-    T56 = bi_vec_t.dot(cA2c) - t**5/30
-    T57 = bi_vec_t.dot(A_cAc) - t**5/40
-    T58 = bi_vec_t.dot(A2c2)/2 - t**5/120
-    T59 = bi_vec_t.dot(A3c) - t**5/120
-
-    # error norm 5 (omitting square root for simplification)
-    T5_norm_t = (T51**2 + T52**2 + T53**2 + T54**2 + T55**2 + T56**2 + T57**2 +
-                 T58**2 + T59**2)
-    T5_norm_i = sympy.integrate(T5_norm_t, (t, 0, 1))
-
-    # minimize norm -> find root of derivative
-    eqns = []
-    for param in free_params:
-        eqns.append(sympy.diff(T5_norm_i, param))
-    if eqns:
-        sol2 = linsolve(eqns, free_params).args[0]
-    else:
-        sol2 = []
-    print('optimal value of free parameters:', sol2)
-
-    # update bi_vec_t and norms
-    subs_dict = dict(zip(free_params, sol2))
-    bi_vec_t = bi_vec_t.subs(subs_dict)
-    T5_norm_t = sympy.sqrt(T5_norm_t.subs(subs_dict))   # now take sqrt
-
-    # create numerical function for plotting
-    T5_fun = sympy.lambdify(t, T5_norm_t, 'numpy')
-    t_ = np.linspace(0., 1., 101)
-    T5_max = T5_fun(t_).max()
-    print('T_5 max:', T5_max)
-    print('T5 max interp/T5 method:', T5_max.max()/T5_method4)
-
-    print('resulting interpolant:')
-    pprint(bi_vec_t)
-
-    # plot error
-    plt.plot(t_, T5_fun(t_), label='free 4th order interpolant')
-    plt.axhline(T5_method4, ls='--', label='embedded 4th order method')
-    plt.tight_layout
-    plt.xlim(0, 1)
-    plt.ylim((0, 6e-4))
-    plt.xlabel(r'$\theta$')
-    plt.ylabel(r'error $\hat{T}_5$')
-    plt.legend(loc=1, ncol=2)
-    plt.title('free interpolant for BS45_i')
-    plt.tight_layout()
-    plt.savefig('free interpolant for BS45_i')
-    plt.show()
+class BS45(BS5):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'BS5'. "
+             "Its option 'interpolant' selects the interpolant",
+             FutureWarning)
+        super(BS45, self).__init__(*args, **kwargs, interpolant="best")
