@@ -1,26 +1,19 @@
 import numpy as np
+from warnings import warn
 from scipy.integrate._ivp.rk import norm, SAFETY
 from extensisq.common import (
     RungeKutta, HornerDenseOutput, CubicDenseOutput, LinearDenseOutput, NFS)
 
 
-class CK45(RungeKutta):
-    """Cash Karp variable order (5, 3, 2) Runge Kutta method with error
-    estimators of order (4, 2, 1). This method is created to efficiently solve
-    non-smooth problems [1]_. Interpolants for dense output have been added.
+class CK5(RungeKutta):
+    """A 5th order method with 4th order error estimator that uses the
+    coefficients of Cash and Karp [1]_.
 
-    The method prefers order 5. Whether this high order can be successfuly
-    reached in the current step is predicted multiple times between the
-    evaluations of the derivative function. After the first failed prediction,
-    propagation with fallback solutions of reduced order and step size is
-    assessed. These fallback solutions do not need extra derivative
-    evaluations.
+    This is not the variable order method described [1]_. That method is
+    available as `CKdisc`.
 
     Can be applied in the complex domain.
 
-    A fixed order (4,5) method with the Cash Karp parameters is available as
-    CK45_o.
-
     Parameters
     ----------
     fun : callable
@@ -57,32 +50,113 @@ class CK45(RungeKutta):
     vectorized : bool, optional
         Whether `fun` is implemented in a vectorized fashion. A vectorized
         implementation offers no advantages for this solver. Default is False.
+    nfev_stiff_detect : int, optional
+        Number of function evaluations for stiffness detection. This number has
+        multiple purposes. If it is set to 0, then stiffness detection is
+        disabled. For other (positive) values it is used to represent a
+        'considerable' number of function evaluations (nfev). A stiffness test
+        is done if many steps fail and each time nfev exceeds integer multiples
+        of `nfev_stiff_detect`. For the assessment itself, the problem is
+        assessed as non-stiff if the predicted nfev to complete the integration
+        is lower than `nfev_stiff_detect`. The default value is 5000.
 
-    Attributes
+    References
     ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
+    .. [1] J. R. Cash, A. H. Karp, "A Variable Order Runge-Kutta Method for
+           Initial Value Problems with Rapidly Varying Right-Hand Sides",
+           ACM Trans. Math. Softw., Vol. 16, No. 3, 1990, pp. 201-222, ISSN
+           0098-3500. https://doi.org/10.1145/79505.79507
+    """
+
+    n_stages = 6
+    order = 5
+    error_estimator_order = 4
+    tanang = 2.4
+    stbrad = 3.7
+
+    A = np.array([
+        [0, 0, 0, 0, 0, 0],
+        [1/5, 0, 0, 0, 0, 0],
+        [3/40, 9/40, 0, 0, 0, 0],
+        [3/10, -9/10, 6/5, 0, 0, 0],
+        [-11/54, 5/2, -70/27, 35/27, 0, 0],
+        [1631/55296, 175/512, 575/13824, 44275/110592, 253/4096, 0]])
+
+    B = np.array([37/378, 0, 250/621, 125/594, 0, 512/1771])
+
+    C = np.array([0, 1/5, 3/10, 3/5, 1, 7/8])
+
+    E = np.array(
+        [277/64512, 0, -6925/370944, 6925/202752, 277/14336, -277/7084, 0])
+
+    # fourth order, maximum ||T5|| over step is 1.52e-3
+    P = np.array([
+        [1, -10405/3843, 32357/11529, -855/854],
+        [0, 0, 0, 0],
+        [0, 308500/88389, -1424000/265167, 67250/29463],
+        [0, 5875/24156, 12875/36234, -3125/8052],
+        [0, 235/1708, -235/854, 235/1708],
+        [0, -287744/108031, 700416/108031, -381440/108031],
+        [0, 3/2, -4, 5/2]])
+
+
+class CKdisc(RungeKutta):
+    """Cash Karp variable order (5, 3, 2) Runge Kutta method with error
+    estimators of order (4, 2, 1). This method is created to efficiently solve
+    non-smooth problems [1]_; problems with discontinuous derivatives.
+    Interpolants for dense output have been added.
+
+    The method prefers order 5. Whether this high order can be successfully
+    reached in the current step is predicted multiple times between the
+    evaluations of the derivative function. After the first failed prediction,
+    propagation with fallback solutions of reduced order and step size is
+    assessed. These fallback solutions do not need extra derivative
+    evaluations.
+
+    Step size is expected to be irregular in this method. This can interfere
+    with stiffness detection, which is therefore disabled.
+
+    Can be applied in the complex domain.
+
+    A fixed fifth order method with the Cash Karp parameters is available as
+    CK5.
+
+    Parameters
+    ----------
+    fun : callable
+        Right-hand side of the system. The calling signature is ``fun(t, y)``.
+        Here ``t`` is a scalar, and there are two options for the ndarray
+        ``y``: It can either have shape (n,); then ``fun`` must return
+        array_like with shape (n,). Alternatively it can have shape (n, k);
+        then ``fun`` must return an array_like with shape (n, k), i.e., each
+        column corresponds to a single column in ``y``. The choice between the
+        two options is determined by `vectorized` argument (see below).
+    t0 : float
+        Initial time.
+    y0 : array_like, shape (n,)
+        Initial state.
     t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
+        Boundary time - the integration won't continue beyond it. It also
+        determines the direction of the integration.
+    first_step : float or None, optional
+        Initial step size. Default is ``None`` which means that the algorithm
+        should choose.
+    max_step : float, optional
+        Maximum allowed step size. Default is np.inf, i.e., the step size is
+        not bounded and determined solely by the solver.
+    rtol, atol : float and array_like, optional
+        Relative and absolute tolerances. The solver keeps the local error
+        estimates less than ``atol + rtol * abs(y)``. Here `rtol` controls a
+        relative accuracy (number of correct digits). But if a component of `y`
+        is approximately below `atol`, the error only needs to fall within
+        the same `atol` threshold, and the number of correct digits is not
+        guaranteed. If components of y have different scales, it might be
+        beneficial to set different `atol` values for different components by
+        passing array_like with shape (n,) for `atol`. Default values are
+        1e-3 for `rtol` and 1e-6 for `atol`.
+    vectorized : bool, optional
+        Whether `fun` is implemented in a vectorized fashion. A vectorized
+        implementation offers no advantages for this solver. Default is False.
 
     References
     ----------
@@ -157,7 +231,8 @@ class CK45(RungeKutta):
         [0, 3/2, -4, 5/2]])
 
     def __init__(self, fun, t0, y0, t_bound, **extraneous):
-        super(CK45, self).__init__(fun, t0, y0, t_bound, **extraneous)
+        super(CKdisc, self).__init__(
+            fun, t0, y0, t_bound, nfev_stiff_detect=0, **extraneous)
         # adaptive weighing factors:
         self.twiddle = [1.5, 1.1]                             # starting values
         self.quit = [100., 100.]                              # starting values
@@ -169,24 +244,35 @@ class CK45(RungeKutta):
         quit = self.quit
 
         # limit step size
-        min_step = 10 * np.abs(np.nextafter(t, self.direction * np.inf) - t)
-        h_abs = min(self.max_step, max(min_step, self.h_abs))
+        h_abs = self.h_abs
+        min_step = max(self.h_min_a * (abs(t) + h_abs), self.h_min_b)
+        h_abs = min(self.max_step, max(min_step, h_abs))
 
-        # use extrapolation in the rare cases where the previous step ended
-        # too close to t_bound.
-        d = abs(self.t_bound - t)
-        if d < min_step:
-            h = self.t_bound - t
-            y_new = y + h * self.f
-            self.h_previous = h
-            self.y_old = y
-            self.t = self.t_bound
-            self.y = y_new
-            self.f = None                          # used by _dense_output_impl
-            return True, None
-
-        # don't step over t_bound
-        h_abs = min(h_abs, d)
+        # handle final integration steps
+        d = abs(self.t_bound - t)               # remaining interval
+        if d < 2 * h_abs:
+            if d >= min_step:
+                if h_abs < d:
+                    # h_abs < d < 2 * h_abs:
+                    # split d over last two steps ("look ahead").
+                    # This reduces the chance of a very small last step.
+                    h_abs = max(0.5 * d, min_step)
+                else:
+                    # d <= h_abs:
+                    # don't step over t_bound
+                    h_abs = d
+            else:
+                # d < min_step:
+                # use linear extrapolation in this rare case
+                h = self.t_bound - t
+                y_new = y + h * self.f
+                self.h_previous = h
+                self.y_old = y
+                self.t = self.t_bound
+                self.y = y_new
+                self.f = None                    # to signal _dense_output_impl
+                warn('\nLinear extrapolation was used in the final step.')
+                return True, None
 
         order_accepted = 0
         step_rejected = False
@@ -330,7 +416,7 @@ class CK45(RungeKutta):
     def _comp_sol_err_tol(self, h, B, E, i=6):
         sol = self._compute_solution(h, B, i)
         err = self._compute_error(h, E, i)
-        tol = self.atol + self.rtol * np.maximum(np.abs(self.y), np.abs(sol))
+        tol = self.atol + self.rtol * 0.5*(np.abs(self.y) + np.abs(sol))
         return sol, err, tol
 
     def _dense_output_impl(self):
@@ -347,268 +433,14 @@ class CK45(RungeKutta):
                                 self.K[0, :], self.K[-1, :])
 
 
-class CK45_o(RungeKutta):
-    """A 5th order method with 4th order error estimator that uses the
-    same coefficients as CK45. In this variant the order is fixed (suffix _o
-    for order).
-
-    Parameters
-    ----------
-    fun : callable
-        Right-hand side of the system. The calling signature is ``fun(t, y)``.
-        Here ``t`` is a scalar, and there are two options for the ndarray
-        ``y``: It can either have shape (n,); then ``fun`` must return
-        array_like with shape (n,). Alternatively it can have shape (n, k);
-        then ``fun`` must return an array_like with shape (n, k), i.e., each
-        column corresponds to a single column in ``y``. The choice between the
-        two options is determined by `vectorized` argument (see below).
-    t0 : float
-        Initial time.
-    y0 : array_like, shape (n,)
-        Initial state.
-    t_bound : float
-        Boundary time - the integration won't continue beyond it. It also
-        determines the direction of the integration.
-    first_step : float or None, optional
-        Initial step size. Default is ``None`` which means that the algorithm
-        should choose.
-    max_step : float, optional
-        Maximum allowed step size. Default is np.inf, i.e., the step size is
-        not bounded and determined solely by the solver.
-    rtol, atol : float and array_like, optional
-        Relative and absolute tolerances. The solver keeps the local error
-        estimates less than ``atol + rtol * abs(y)``. Here `rtol` controls a
-        relative accuracy (number of correct digits). But if a component of `y`
-        is approximately below `atol`, the error only needs to fall within
-        the same `atol` threshold, and the number of correct digits is not
-        guaranteed. If components of y have different scales, it might be
-        beneficial to set different `atol` values for different components by
-        passing array_like with shape (n,) for `atol`. Default values are
-        1e-3 for `rtol` and 1e-6 for `atol`.
-    vectorized : bool, optional
-        Whether `fun` is implemented in a vectorized fashion. A vectorized
-        implementation offers no advantages for this solver. Default is False.
-
-    Attributes
-    ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
-    t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
-
-    References
-    ----------
-    .. [1] J. R. Cash, A. H. Karp, "A Variable Order Runge-Kutta Method for
-           Initial Value Problems with Rapidly Varying Right-Hand Sides",
-           ACM Trans. Math. Softw., Vol. 16, No. 3, 1990, pp. 201-222, ISSN
-           0098-3500. https://doi.org/10.1145/79505.79507
-    """
-
-    n_stages = CK45.n_stages
-    order = CK45.order
-    error_estimator_order = CK45.error_estimator_order
-
-    A = CK45.A
-    B = CK45.B
-    C = CK45.C
-    P = CK45.P
-    E = CK45.E
+# old class names
+class CK45_o(CK5):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'CK5'.", FutureWarning)
+        super(CK45_o, self).__init__(*args, **kwargs)
 
 
-if __name__ == '__main__':
-    """Construction of a free interpolant of the CK45 pair. The approach from
-    "Runge-Kutta pairs of order 5(4) satisfying only the first column
-    simplifying assumption" by Ch Tsitouras is followed."""
-    # import numpy as np
-    import matplotlib.pyplot as plt
-    import sympy
-    from sympy.solvers.solveset import linsolve
-    from sympy import Rational as R
-    from pprint import pprint
-
-    n_stages = 7         # including derivative evaluation at end of step
-    order = 4            # of interpolation in t (not h)
-    T5_method4 = 5e-4    # error of embedded fourth order method
-
-    t = sympy.symbols('t', real=True)
-    bi = sympy.symbols(f'bi0:{n_stages}', real=True)
-    bi_vec = sympy.Matrix(bi)
-
-    # Method
-    A = sympy.Matrix([
-            [0, 0, 0, 0, 0, 0, 0],
-            [R(1, 5), 0, 0, 0, 0, 0, 0],
-            [R(3, 40), R(9, 40), 0, 0, 0, 0, 0],
-            [R(3, 10), R(-9, 10), R(6, 5), 0, 0, 0, 0],
-            [R(-11, 54), R(5, 2), R(-70, 27), R(35, 27), 0, 0, 0],
-            [R(1631, 55296), R(175, 512), R(575, 13824), R(44275, 110592),
-             R(253, 4096), 0, 0],
-            [R(37, 378), 0, R(250, 621), R(125, 594), 0, R(512, 1771), 0]
-        ])     # output stage appended
-
-    c = sympy.Matrix([0, R(1, 5), R(3, 10), R(3, 5), 1, R(7, 8), 1])
-    e = sympy.Matrix([1, 1, 1, 1, 1, 1, 1])
-
-    # error terms up to order 4
-    c2 = c.multiply_elementwise(c)
-    Ac = A*c
-
-    c3 = c2.multiply_elementwise(c)
-    cAc = c.multiply_elementwise(Ac)
-    Ac2 = A*c2
-    A2c = A*Ac
-
-    T11 = bi_vec.dot(e) - t
-    T21 = bi_vec.dot(c) - t**2/2
-
-    T31 = bi_vec.dot(c2)/2 - t**3/6
-    T32 = bi_vec.dot(Ac) - t**3/6
-
-    T41 = bi_vec.dot(c3)/6 - t**4/24
-    T42 = bi_vec.dot(cAc) - t**4/8
-    T43 = bi_vec.dot(Ac2)/2 - t**4/24
-    T44 = bi_vec.dot(A2c) - t**4/24
-
-    # solve polynomials to let all terms up to order 4 vanish
-    bi_vec_t = sympy.Matrix(
-        linsolve([T11, T21, T31, T32, T41, T42, T43, T44], bi).args[0])
-    i_free_poly = [i for i, (term, poly) in enumerate(zip(bi_vec_t, bi))
-                   if poly == term]
-    free_polys = [bi[i] for i in i_free_poly]
-    print('free polynomials:', free_polys)      # poly bi5 is free to define
-
-    # Make this free polynommial explicit in t
-    parameters = sympy.symbols([f'bi{i}_0:{order+1}' for i in i_free_poly])
-    polys = []
-    for coefs in parameters:
-        p = 0
-        for i, coef in enumerate(coefs):
-            p = p + coef * t**i
-        polys.append(p)
-
-    # substitute in bi_vec_t
-    subs_dict = dict(zip(free_polys, polys))
-    bi_vec_t = bi_vec_t.subs(subs_dict)
-
-    # demand continuity at start and end of step
-    d_bi_vec_t = sympy.diff(bi_vec_t, t)        # derivative
-    # C0 at t=0
-    C0_0 = [eq for eq in bi_vec_t.subs(t, 0)]
-    # C0 at t=1
-    C0_1 = [eq for eq in (bi_vec_t.subs(t, 1) - A[-1, :].T)]
-    # C1 at t=0
-    C1_0 = d_bi_vec_t.subs(t, 0)
-    C1_0[0] = C1_0[0] - 1
-    C1_0 = [eq for eq in C1_0]
-    # C1 at t=1
-    C1_1 = d_bi_vec_t.subs(t, 1)
-    C1_1[-1] = C1_1[-1] - 1
-    C1_1 = [eq for eq in C1_1]
-
-    # combine equations in list
-    eqns = C0_0
-    eqns.extend(C0_1)
-    eqns.extend(C1_0)
-    eqns.extend(C1_1)
-
-    # combine parameters in list
-    params = []
-    for p in parameters:
-        params.extend(p)
-
-    # solve continuity constraints
-    sol1 = linsolve(eqns, params).args[0]
-
-    # whych params are still free?
-    free_params = [p for s, p in zip(sol1, params) if s == p]
-    print('free parameters:', free_params)      # free parameter: bi5_4
-
-    # update bi_vec_t
-    subs_dict = dict(zip(params, sol1))
-    bi_vec_t = bi_vec_t.subs(subs_dict)
-
-    # find value for free parameter that minimizes the 5th order error terms
-
-    # error terms of order 5
-    c4 = c3.multiply_elementwise(c)
-    c2Ac = c2.multiply_elementwise(Ac)
-    Ac_Ac = Ac.multiply_elementwise(Ac)
-    cAc2 = c.multiply_elementwise(Ac2)
-    Ac3 = A*c3
-    cA2c = c.multiply_elementwise(A2c)
-    A_cAc = A*cAc
-    A2c2 = A*Ac2
-    A3c = A*A2c
-
-    T51 = bi_vec_t.dot(c4)/24 - t**5/120
-    T52 = bi_vec_t.dot(c2Ac)/2 - t**5/20
-    T53 = bi_vec_t.dot(Ac_Ac)/2 - t**5/40
-    T54 = bi_vec_t.dot(cAc2)/2 - t**5/30
-    T55 = bi_vec_t.dot(Ac3)/6 - t**5/120
-    T56 = bi_vec_t.dot(cA2c) - t**5/30
-    T57 = bi_vec_t.dot(A_cAc) - t**5/40
-    T58 = bi_vec_t.dot(A2c2)/2 - t**5/120
-    T59 = bi_vec_t.dot(A3c) - t**5/120
-
-    # error norm 5 (omitting square root for simplification)
-    T5_norm_t = (T51**2 + T52**2 + T53**2 + T54**2 + T55**2 + T56**2 + T57**2 +
-                 T58**2 + T59**2)
-    T5_norm_i = sympy.integrate(T5_norm_t, (t, 0, 1))
-
-    # minimize norm -> find root of derivative
-    eqns = []
-    for param in free_params:
-        eqns.append(sympy.diff(T5_norm_i, param))
-    if eqns:
-        sol2 = linsolve(eqns, free_params).args[0]
-    else:
-        sol2 = []
-    print('optimal value of free parameters:', sol2)
-
-    # update bi_vec_t and norms
-    subs_dict = dict(zip(free_params, sol2))
-    bi_vec_t = bi_vec_t.subs(subs_dict)
-    T5_norm_t = sympy.sqrt(T5_norm_t.subs(subs_dict))   # now take sqrt
-
-    # create numerical function for plotting
-    T5_fun = sympy.lambdify(t, T5_norm_t, 'numpy')
-    t_ = np.linspace(0., 1., 101)
-    T5_max = T5_fun(t_).max()
-    print('T_5 max:', T5_max)
-    print('T5 max interp/T5 method:', T5_max.max()/T5_method4)
-
-    print('resulting interpolant:')
-    pprint(bi_vec_t)
-
-    # plot error
-    plt.plot(t_, T5_fun(t_), label='free 4th order interpolant')
-    plt.axhline(T5_method4, ls='--', label='embedded 4th order method')
-    plt.tight_layout
-    plt.xlim(0, 1)
-    plt.ylim(ymin=0)
-    plt.xlabel(r'$\theta$')
-    plt.ylabel(r'error $\hat{T}_5$')
-    plt.legend(loc=1)
-    plt.title('free interpolant for CK45')
-    plt.tight_layout()
-    plt.savefig('free interpolant for CK45')
-    plt.show()
+class CK45(CKdisc):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'CKdisc'.", FutureWarning)
+        super(CK45, self).__init__(*args, **kwargs)
