@@ -1,8 +1,9 @@
 import numpy as np
+from warnings import warn
 from extensisq.common import RungeKutta
 
 
-class Pri6(RungeKutta):
+class Pr7(RungeKutta):
     """Explicit Runge-Kutta method by Prince [1]_, developed around a
     continuous method of order 6. Discrete propagation is of order 7 and the
     solution for error estimation and stepsize selection is of order 5.
@@ -45,32 +46,15 @@ class Pri6(RungeKutta):
     vectorized : bool, optional
         Whether `fun` is implemented in a vectorized fashion. A vectorized
         implementation offers no advantages for this solver. Default is False.
-
-    Attributes
-    ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
-    t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
+    nfev_stiff_detect : int, optional
+        Number of function evaluations for stiffness detection. This number has
+        multiple purposes. If it is set to 0, then stiffness detection is
+        disabled. For other (positive) values it is used to represent a
+        'considerable' number of function evaluations (nfev). A stiffness test
+        is done if many steps fail and each time nfev exceeds integer multiple
+        of `nfev_stiff_detect`. For the assessment itself, the problem is
+        assessed as non-stiff if the predicted nfev to complete the integration
+        is lower than `nfev_stiff_detect`. The default value is 5000.
 
     References
     ----------
@@ -83,6 +67,8 @@ class Pri6(RungeKutta):
     order = 7
     error_estimator_order = 5
     n_stages = 10
+    tanang = 7.5
+    stbrad = 4.1
 
     C = np.array([0, 1/6, 1/4, 1/2, 1/2, 3/16, 3/16, 3/5, 6/7, 1])
 
@@ -129,7 +115,7 @@ class Pri6(RungeKutta):
     ])
 
 
-class Pri7(RungeKutta):
+class Pr8(RungeKutta):
     """Explicit Runge-Kutta method by Prince [1]_, developed around a
     continuous method of order 7. Discrete propagation is of order 8 and the
     solution for error estimation and stepsize selection is of order 6.
@@ -172,32 +158,15 @@ class Pri7(RungeKutta):
     vectorized : bool, optional
         Whether `fun` is implemented in a vectorized fashion. A vectorized
         implementation offers no advantages for this solver. Default is False.
-
-    Attributes
-    ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
-    t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
+    nfev_stiff_detect : int, optional
+        Number of function evaluations for stiffness detection. This number has
+        multiple purposes. If it is set to 0, then stiffness detection is
+        disabled. For other (positive) values it is used to represent a
+        'considerable' number of function evaluations (nfev). A stiffness test
+        is done if many steps fail and each time nfev exceeds integer multiples
+        of `nfev_stiff_detect`. For the assessment itself, the problem is
+        assessed as non-stiff if the predicted nfev to complete the integration
+        is lower than `nfev_stiff_detect`. The default value is 5000.
 
     References
     ----------
@@ -210,6 +179,8 @@ class Pri7(RungeKutta):
     order = 8
     error_estimator_order = 6
     n_stages = 13
+    tanang = 6.0
+    stbrad = 4.5
 
     A = np.array([
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
@@ -282,7 +253,8 @@ class Pri7(RungeKutta):
          4.52499466462306516458275969143E-1, 0],
     ])
 
-    C = np.array(A.sum(axis=1))
+    C = np.array([0, 7/75, 7/50, 7/25, 7/25, 19/40, 19/40, 7/50, 2/25, 8/15,
+                  4/5, 22/25, 1])
 
     B = np.array([
         1.01794429109647960346134352579E-2, 0, 0, 0,
@@ -373,7 +345,7 @@ class Pri7(RungeKutta):
     ])
 
 
-class Pri8(RungeKutta):
+class Pr9(RungeKutta):
     """Explicit Runge-Kutta method by Prince [1]_, developed around a
     continuous method of order 8. Discrete propagation is of order 9 and the
     solution for error estimation and stepsize selection is of order 7.
@@ -416,32 +388,15 @@ class Pri8(RungeKutta):
     vectorized : bool, optional
         Whether `fun` is implemented in a vectorized fashion. A vectorized
         implementation offers no advantages for this solver. Default is False.
-
-    Attributes
-    ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
-    t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
+    nfev_stiff_detect : int, optional
+        Number of function evaluations for stiffness detection. This number has
+        multiple purposes. If it is set to 0, then stiffness detection is
+        disabled. For other (positive) values it is used to represent a
+        'considerable' number of function evaluations (nfev). A stiffness test
+        is done if many steps fail and each time nfev exceeds integer multiples
+        of `nfev_stiff_detect`. For the assessment itself, the problem is
+        assessed as non-stiff if the predicted nfev to complete the integration
+        is lower than `nfev_stiff_detect`. The default value is 5000.
 
     References
     ----------
@@ -454,6 +409,8 @@ class Pri8(RungeKutta):
     order = 9
     error_estimator_order = 7
     n_stages = 17
+    tanang = 7.5
+    stbrad = 4.6
 
     A = np.array([
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
@@ -609,7 +566,8 @@ class Pri8(RungeKutta):
             0],
     ])
 
-    C = np.array(A.sum(axis=1))
+    C = np.array([0, 14/135, 7/45, 14/45, 14/45, 29/45, 29/45, 7/45, 3/4, 1/50,
+                  13/15, 0.36, 2/21,  22/45, 7/11, 14/15, 1])
 
     B = np.array([
         3.07799270475412360737919532866E-2,
@@ -745,3 +703,22 @@ class Pri8(RungeKutta):
             -7.86288962134219494870125843802E1,
             2.70849507236009171823909477318E1],
     ])
+
+
+# old class names
+class Pri6(Pr7):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'Pr7'.", FutureWarning)
+        super(Pri6, self).__init__(*args, **kwargs)
+
+
+class Pri7(Pr8):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'Pr8'.", FutureWarning)
+        super(Pri7, self).__init__(*args, **kwargs)
+
+
+class Pri8(Pr9):
+    def __init__(self, *args, **kwargs):
+        warn("This method will be replaced by 'Pr9'.", FutureWarning)
+        super(Pri8, self).__init__(*args, **kwargs)
