@@ -2,10 +2,9 @@ import numpy as np
 from warnings import warn
 from math import copysign, sqrt
 from scipy.integrate._ivp.base import OdeSolver, DenseOutput
-from extensisq.common import h_start, LinearDenseOutput, NFS
+from extensisq.common import h_start, LinearDenseOutput, validate_tol, NFS
 from scipy.integrate._ivp.common import (
-    validate_max_step, validate_tol, norm, warn_extraneous,
-    validate_first_step)
+    validate_max_step, norm, warn_extraneous, validate_first_step)
 
 
 class SWAG(OdeSolver):
@@ -90,7 +89,7 @@ class SWAG(OdeSolver):
            https://www.netlib.org/slatec/src/
     .. [2] L.F. Shampine and M.K. Gordon, "Computer solution of ordinary
            differential equations: The initial value problem", San Francisco,
-           W.H. Freeman.
+           W.H. Freeman, 1975.
     .. [3] H.A. Watts and L.F. Shampine, "Smoother Interpolants for Adams
            Codes",  SIAM Journal on Scientific and Statistical Computing, 1986,
            Vol. 7, No. 1, pp. 334-345. ISSN 0196-5204
@@ -106,12 +105,14 @@ class SWAG(OdeSolver):
         super(SWAG, self).__init__(
             fun, t0, y0, t_bound, vectorized, support_complex=True)
         self.max_step = validate_max_step(max_step)
-        self.rtol, self.atol = validate_tol(rtol, atol, self.n)
+        self.rtol, self.atol = validate_tol(rtol, atol, self.y)
 
         # starting step size
         self.yp = self.fun(self.t, self.y)                 # initial evaluation
         if first_step is None:
-            self.h = h_start(self.fun, self.t, self.t_bound, self.y, self.yp,
+            b = self.t + copysign(min(abs(self.t_bound - self.t),
+                                      self.max_step), self.direction)
+            self.h = h_start(self.fun, self.t, b, self.y, self.yp,
                              1, self.rtol, self.atol)
         else:
             h_abs = validate_first_step(first_step, t0, t_bound)
@@ -145,8 +146,8 @@ class SWAG(OdeSolver):
 
         # Tolerances are dealt with like in scipy: wt is like scipy's scale
         # and will be update each step.  This is only the initial value:
-        self.wt = self.atol + self.rtol * np.maximum(
-            np.abs(self.y), np.abs(self.y - self.h*self.yp))
+        self.wt = self.atol + self.rtol * 0.5*(
+            np.abs(self.y) + np.abs(self.y - self.h*self.yp))
 
         # initialization
         # from  *** block 0 ***  of dsteps.f, under IF START:
@@ -157,7 +158,8 @@ class SWAG(OdeSolver):
             # The compensated summation of the original code that would be
             # executed if nornd == False has been removed.  Instead, this
             # warning is given to the user.
-            warn("numerical rounding may limit accuracy at this tolerance.")
+            warn("Numerical rounding may limit the accuracy "
+                 "at this tolerance.")
         self.phi[:, 0] = self.yp
         self.phi[:, 1] = 0.0
         self.sig[0] = 1.0
@@ -173,7 +175,6 @@ class SWAG(OdeSolver):
         self.ns = 0
 
         # from ddes.f, for stiffness detection
-        self.stiff = False
         self.kle4 = 0
 
     def _step_impl(self):
@@ -199,11 +200,11 @@ class SWAG(OdeSolver):
             self.kle4 = 0
         else:
             self.kle4 += 1
-            if self.kle4 > 50 and not self.stiff and self.k_max > 4:
+            if self.kle4 > 50 and self.k_max > 4:
                 # This warning is issued once, after 50 consequtive steps are
                 # taken with order <= 4, while k_max > 4.
-                self.stiff = True
-                warn("problem appears to be stiff (for this tolerance).")
+                warn("Your problem appears to be stiff (for this tolerance).")
+                self.kle4 = 0
 
         # extrapolate if too close to t_bound
         d = self.t_bound - x
@@ -226,7 +227,6 @@ class SWAG(OdeSolver):
 
         # (***first executable statement dsteps)
         if abs(h) < min_step:
-            h = copysign(min_step, h)
             return False, self.TOO_SMALL_STEP
 
         # If error tolerance is too small, increase it to an acceptable value
@@ -341,7 +341,7 @@ class SWAG(OdeSolver):
             yp[:] = self.fun(x, p)                                   # evaluate
 
             # added update of wt:
-            wt[:] = self.atol + self.rtol * np.maximum(np.abs(p), np.abs(y))
+            wt[:] = self.atol + self.rtol * 0.5*(np.abs(p) + np.abs(y))
 
             # estimate errors at orders k, k-1, k-2
             temp3 = 1.0 / wt
